@@ -2,18 +2,18 @@
 // SPDX-License-Identifier: Apache-2.0
 
 //! The protocol for vhost-user is based on the existing implementation of vhost for the Linux
-//! Kernel. The protocol defines two sides of the communication, master and slave. Master is
-//! the application that shares its virtqueues. Slave is the consumer of the virtqueues.
+//! Kernel. The protocol defines two sides of the communication, frontend and backend. Frontend is
+//! the application that shares its virtqueues. Backend is the consumer of the virtqueues.
 //!
-//! The communication channel between the master and the slave includes two sub channels. One is
-//! used to send requests from the master to the slave and optional replies from the slave to the
-//! master. This sub channel is created on master startup by connecting to the slave service
-//! endpoint. The other is used to send requests from the slave to the master and optional replies
-//! from the master to the slave. This sub channel is created by the master issuing a
-//! VHOST_USER_SET_SLAVE_REQ_FD request to the slave with an auxiliary file descriptor.
+//! The communication channel between the frontend and the backend includes two sub channels. One is
+//! used to send requests from the frontend to the backend and optional replies from the backend to the
+//! frontend. This sub channel is created on frontend startup by connecting to the backend service
+//! endpoint. The other is used to send requests from the backend to the frontend and optional replies
+//! from the frontend to the backend. This sub channel is created by the frontend issuing a
+//! VHOST_USER_SET_BACKEND_REQ_FD request to the backend with an auxiliary file descriptor.
 //!
-//! Unix domain socket is used as the underlying communication channel because the master needs to
-//! send file descriptors to the slave.
+//! Unix domain socket is used as the underlying communication channel because the frontend needs to
+//! send file descriptors to the backend.
 //!
 //! Most messages that can be sent via the Unix domain socket implementing vhost-user have an
 //! equivalent ioctl to the kernel implementation.
@@ -27,31 +27,31 @@ pub use self::message::{VhostUserProtocolFeatures, VhostUserVirtioFeatures};
 mod connection;
 pub use self::connection::Listener;
 
-#[cfg(feature = "vhost-user-master")]
-mod master;
-#[cfg(feature = "vhost-user-master")]
-pub use self::master::{Master, VhostUserMaster};
+#[cfg(feature = "vhost-user-frontend")]
+mod frontend;
+#[cfg(feature = "vhost-user-frontend")]
+pub use self::frontend::{Frontend, VhostUserFrontend};
 #[cfg(feature = "vhost-user")]
-mod master_req_handler;
+mod frontend_req_handler;
 #[cfg(feature = "vhost-user")]
-pub use self::master_req_handler::{
-    MasterReqHandler, VhostUserMasterReqHandler, VhostUserMasterReqHandlerMut,
+pub use self::frontend_req_handler::{
+    FrontendReqHandler, VhostUserFrontendReqHandler, VhostUserFrontendReqHandlerMut,
 };
 
-#[cfg(feature = "vhost-user-slave")]
-mod slave;
-#[cfg(feature = "vhost-user-slave")]
-pub use self::slave::SlaveListener;
-#[cfg(feature = "vhost-user-slave")]
-mod slave_req_handler;
-#[cfg(feature = "vhost-user-slave")]
-pub use self::slave_req_handler::{
-    SlaveReqHandler, VhostUserSlaveReqHandler, VhostUserSlaveReqHandlerMut,
+#[cfg(feature = "vhost-user-backend")]
+mod backend;
+#[cfg(feature = "vhost-user-backend")]
+pub use self::backend::BackendListener;
+#[cfg(feature = "vhost-user-backend")]
+mod backend_req_handler;
+#[cfg(feature = "vhost-user-backend")]
+pub use self::backend_req_handler::{
+    BackendReqHandler, VhostUserBackendReqHandler, VhostUserBackendReqHandlerMut,
 };
-#[cfg(feature = "vhost-user-slave")]
-mod slave_req;
-#[cfg(feature = "vhost-user-slave")]
-pub use self::slave_req::Slave;
+#[cfg(feature = "vhost-user-backend")]
+mod backend_req;
+#[cfg(feature = "vhost-user-backend")]
+pub use self::backend_req::Backend;
 
 /// Errors for vhost-user operations
 #[derive(Debug)]
@@ -82,10 +82,10 @@ pub enum Error {
     SocketBroken(std::io::Error),
     /// Should retry the socket operation again.
     SocketRetry(std::io::Error),
-    /// Failure from the slave side.
-    SlaveInternalError,
-    /// Failure from the master side.
-    MasterInternalError,
+    /// Failure from the backend side.
+    BackendInternalError,
+    /// Failure from the frontend side.
+    FrontendInternalError,
     /// Virtio/protocol features mismatch.
     FeatureMismatch,
     /// Error from request handler
@@ -116,8 +116,8 @@ impl std::fmt::Display for Error {
             Error::SocketConnect(e) => write!(f, "can't connect to peer: {}", e),
             Error::SocketBroken(e) => write!(f, "socket is broken: {}", e),
             Error::SocketRetry(e) => write!(f, "temporary socket error: {}", e),
-            Error::SlaveInternalError => write!(f, "slave internal error"),
-            Error::MasterInternalError => write!(f, "Master internal error"),
+            Error::BackendInternalError => write!(f, "backend internal error"),
+            Error::FrontendInternalError => write!(f, "Frontend internal error"),
             Error::FeatureMismatch => write!(f, "virtio/protocol features mismatch"),
             Error::ReqHandlerError(e) => write!(f, "handler failed to handle request: {}", e),
             Error::MemFdCreateError => {
@@ -144,10 +144,10 @@ impl Error {
             Error::PartialMessage => true,
             // Should reconnect because the underline socket is broken.
             Error::SocketBroken(_) => true,
-            // Slave internal error, hope it recovers on reconnect.
-            Error::SlaveInternalError => true,
-            // Master internal error, hope it recovers on reconnect.
-            Error::MasterInternalError => true,
+            // Backend internal error, hope it recovers on reconnect.
+            Error::BackendInternalError => true,
+            // Frontend internal error, hope it recovers on reconnect.
+            Error::FrontendInternalError => true,
             // Should just retry the IO operation instead of rebuilding the underline connection.
             Error::SocketRetry(_) => false,
             // Looks like the peer deliberately disconnected the socket.
@@ -218,10 +218,10 @@ pub(crate) fn take_single_file(files: Option<Vec<File>>) -> Option<File> {
     Some(files.swap_remove(0))
 }
 
-#[cfg(all(test, feature = "vhost-user-slave"))]
-mod dummy_slave;
+#[cfg(all(test, feature = "vhost-user-backend"))]
+mod dummy_backend;
 
-#[cfg(all(test, feature = "vhost-user-master", feature = "vhost-user-slave"))]
+#[cfg(all(test, feature = "vhost-user-frontend", feature = "vhost-user-backend"))]
 mod tests {
     use std::fs::File;
     use std::os::unix::io::AsRawFd;
@@ -231,7 +231,7 @@ mod tests {
     use vmm_sys_util::rand::rand_alphanumerics;
     use vmm_sys_util::tempfile::TempFile;
 
-    use super::dummy_slave::{DummySlaveReqHandler, VIRTIO_FEATURES};
+    use super::dummy_backend::{DummyBackendReqHandler, VIRTIO_FEATURES};
     use super::message::*;
     use super::*;
     use crate::backend::VhostBackend;
@@ -244,38 +244,38 @@ mod tests {
         ))
     }
 
-    fn create_slave<P, S>(path: P, backend: Arc<S>) -> (Master, SlaveReqHandler<S>)
+    fn create_backend<P, S>(path: P, backend: Arc<S>) -> (Frontend, BackendReqHandler<S>)
     where
         P: AsRef<Path>,
-        S: VhostUserSlaveReqHandler,
+        S: VhostUserBackendReqHandler,
     {
         let listener = Listener::new(&path, true).unwrap();
-        let mut slave_listener = SlaveListener::new(listener, backend).unwrap();
-        let master = Master::connect(&path, 1).unwrap();
-        (master, slave_listener.accept().unwrap().unwrap())
+        let mut backend_listener = BackendListener::new(listener, backend).unwrap();
+        let frontend = Frontend::connect(&path, 1).unwrap();
+        (frontend, backend_listener.accept().unwrap().unwrap())
     }
 
     #[test]
-    fn create_dummy_slave() {
-        let slave = Arc::new(Mutex::new(DummySlaveReqHandler::new()));
+    fn create_dummy_backend() {
+        let backend = Arc::new(Mutex::new(DummyBackendReqHandler::new()));
 
-        slave.set_owner().unwrap();
-        assert!(slave.set_owner().is_err());
+        backend.set_owner().unwrap();
+        assert!(backend.set_owner().is_err());
     }
 
     #[test]
     fn test_set_owner() {
-        let slave_be = Arc::new(Mutex::new(DummySlaveReqHandler::new()));
+        let backend_be = Arc::new(Mutex::new(DummyBackendReqHandler::new()));
         let path = temp_path();
-        let (master, mut slave) = create_slave(path, slave_be.clone());
+        let (frontend, mut backend) = create_backend(path, backend_be.clone());
 
-        assert!(!slave_be.lock().unwrap().owned);
-        master.set_owner().unwrap();
-        slave.handle_request().unwrap();
-        assert!(slave_be.lock().unwrap().owned);
-        master.set_owner().unwrap();
-        assert!(slave.handle_request().is_err());
-        assert!(slave_be.lock().unwrap().owned);
+        assert!(!backend_be.lock().unwrap().owned);
+        frontend.set_owner().unwrap();
+        backend.handle_request().unwrap();
+        assert!(backend_be.lock().unwrap().owned);
+        frontend.set_owner().unwrap();
+        assert!(backend.handle_request().is_err());
+        assert!(backend_be.lock().unwrap().owned);
     }
 
     #[test]
@@ -283,68 +283,68 @@ mod tests {
         let mbar = Arc::new(Barrier::new(2));
         let sbar = mbar.clone();
         let path = temp_path();
-        let slave_be = Arc::new(Mutex::new(DummySlaveReqHandler::new()));
-        let (mut master, mut slave) = create_slave(path, slave_be.clone());
+        let backend_be = Arc::new(Mutex::new(DummyBackendReqHandler::new()));
+        let (mut frontend, mut backend) = create_backend(path, backend_be.clone());
 
         thread::spawn(move || {
-            slave.handle_request().unwrap();
-            assert!(slave_be.lock().unwrap().owned);
+            backend.handle_request().unwrap();
+            assert!(backend_be.lock().unwrap().owned);
 
-            slave.handle_request().unwrap();
-            slave.handle_request().unwrap();
+            backend.handle_request().unwrap();
+            backend.handle_request().unwrap();
             assert_eq!(
-                slave_be.lock().unwrap().acked_features,
+                backend_be.lock().unwrap().acked_features,
                 VIRTIO_FEATURES & !0x1
             );
 
-            slave.handle_request().unwrap();
-            slave.handle_request().unwrap();
+            backend.handle_request().unwrap();
+            backend.handle_request().unwrap();
             assert_eq!(
-                slave_be.lock().unwrap().acked_protocol_features,
+                backend_be.lock().unwrap().acked_protocol_features,
                 VhostUserProtocolFeatures::all().bits()
             );
 
             sbar.wait();
         });
 
-        master.set_owner().unwrap();
+        frontend.set_owner().unwrap();
 
         // set virtio features
-        let features = master.get_features().unwrap();
+        let features = frontend.get_features().unwrap();
         assert_eq!(features, VIRTIO_FEATURES);
-        master.set_features(VIRTIO_FEATURES & !0x1).unwrap();
+        frontend.set_features(VIRTIO_FEATURES & !0x1).unwrap();
 
         // set vhost protocol features
-        let features = master.get_protocol_features().unwrap();
+        let features = frontend.get_protocol_features().unwrap();
         assert_eq!(features.bits(), VhostUserProtocolFeatures::all().bits());
-        master.set_protocol_features(features).unwrap();
+        frontend.set_protocol_features(features).unwrap();
 
         mbar.wait();
     }
 
     #[test]
-    fn test_master_slave_process() {
+    fn test_frontend_backend_process() {
         let mbar = Arc::new(Barrier::new(2));
         let sbar = mbar.clone();
         let path = temp_path();
-        let slave_be = Arc::new(Mutex::new(DummySlaveReqHandler::new()));
-        let (mut master, mut slave) = create_slave(path, slave_be.clone());
+        let backend_be = Arc::new(Mutex::new(DummyBackendReqHandler::new()));
+        let (mut frontend, mut backend) = create_backend(path, backend_be.clone());
 
         thread::spawn(move || {
             // set_own()
-            slave.handle_request().unwrap();
-            assert!(slave_be.lock().unwrap().owned);
+            backend.handle_request().unwrap();
+            assert!(backend_be.lock().unwrap().owned);
 
             // get/set_features()
-            slave.handle_request().unwrap();
-            slave.handle_request().unwrap();
+            backend.handle_request().unwrap();
+            backend.handle_request().unwrap();
             assert_eq!(
-                slave_be.lock().unwrap().acked_features,
+                backend_be.lock().unwrap().acked_features,
                 VIRTIO_FEATURES & !0x1
             );
 
-            slave.handle_request().unwrap();
-            slave.handle_request().unwrap();
+            backend.handle_request().unwrap();
+            backend.handle_request().unwrap();
 
             let mut features = VhostUserProtocolFeatures::all();
 
@@ -354,64 +354,64 @@ mod tests {
             }
 
             assert_eq!(
-                slave_be.lock().unwrap().acked_protocol_features,
+                backend_be.lock().unwrap().acked_protocol_features,
                 features.bits()
             );
 
             // get_inflight_fd()
-            slave.handle_request().unwrap();
+            backend.handle_request().unwrap();
             // set_inflight_fd()
-            slave.handle_request().unwrap();
+            backend.handle_request().unwrap();
 
             // get_queue_num()
-            slave.handle_request().unwrap();
+            backend.handle_request().unwrap();
 
             // set_mem_table()
-            slave.handle_request().unwrap();
+            backend.handle_request().unwrap();
 
             // get/set_config()
-            slave.handle_request().unwrap();
-            slave.handle_request().unwrap();
+            backend.handle_request().unwrap();
+            backend.handle_request().unwrap();
 
-            // set_slave_request_fd
-            slave.handle_request().unwrap();
+            // set_backend_request_fd
+            backend.handle_request().unwrap();
 
             // set_vring_enable
-            slave.handle_request().unwrap();
+            backend.handle_request().unwrap();
 
             // set_log_base,set_log_fd()
-            slave.handle_request().unwrap_err();
-            slave.handle_request().unwrap_err();
+            backend.handle_request().unwrap_err();
+            backend.handle_request().unwrap_err();
 
             // set_vring_xxx
-            slave.handle_request().unwrap();
-            slave.handle_request().unwrap();
-            slave.handle_request().unwrap();
-            slave.handle_request().unwrap();
-            slave.handle_request().unwrap();
-            slave.handle_request().unwrap();
+            backend.handle_request().unwrap();
+            backend.handle_request().unwrap();
+            backend.handle_request().unwrap();
+            backend.handle_request().unwrap();
+            backend.handle_request().unwrap();
+            backend.handle_request().unwrap();
 
             // get_max_mem_slots()
-            slave.handle_request().unwrap();
+            backend.handle_request().unwrap();
 
             // add_mem_region()
-            slave.handle_request().unwrap();
+            backend.handle_request().unwrap();
 
             // remove_mem_region()
-            slave.handle_request().unwrap();
+            backend.handle_request().unwrap();
 
             sbar.wait();
         });
 
-        master.set_owner().unwrap();
+        frontend.set_owner().unwrap();
 
         // set virtio features
-        let features = master.get_features().unwrap();
+        let features = frontend.get_features().unwrap();
         assert_eq!(features, VIRTIO_FEATURES);
-        master.set_features(VIRTIO_FEATURES & !0x1).unwrap();
+        frontend.set_features(VIRTIO_FEATURES & !0x1).unwrap();
 
         // set vhost protocol features
-        let mut features = master.get_protocol_features().unwrap();
+        let mut features = frontend.get_protocol_features().unwrap();
         assert_eq!(features.bits(), VhostUserProtocolFeatures::all().bits());
 
         // Disable Xen mmap feature.
@@ -419,10 +419,10 @@ mod tests {
             features.remove(VhostUserProtocolFeatures::XEN_MMAP);
         }
 
-        master.set_protocol_features(features).unwrap();
+        frontend.set_protocol_features(features).unwrap();
 
         // Retrieve inflight I/O tracking information
-        let (inflight_info, inflight_file) = master
+        let (inflight_info, inflight_file) = frontend
             .get_inflight_fd(&VhostUserInflight {
                 num_queues: 2,
                 queue_size: 256,
@@ -430,11 +430,11 @@ mod tests {
             })
             .unwrap();
         // Set the buffer back to the backend
-        master
+        frontend
             .set_inflight_fd(&inflight_info, inflight_file.as_raw_fd())
             .unwrap();
 
-        let num = master.get_queue_num().unwrap();
+        let num = frontend.get_queue_num().unwrap();
         assert_eq!(num, 2);
 
         let eventfd = vmm_sys_util::eventfd::EventFd::new(0).unwrap();
@@ -445,23 +445,23 @@ mod tests {
             0,
             eventfd.as_raw_fd(),
         )];
-        master.set_mem_table(&mem).unwrap();
+        frontend.set_mem_table(&mem).unwrap();
 
-        master
+        frontend
             .set_config(0x100, VhostUserConfigFlags::WRITABLE, &[0xa5u8; 4])
             .unwrap();
         let buf = [0x0u8; 4];
-        let (reply_body, reply_payload) = master
+        let (reply_body, reply_payload) = frontend
             .get_config(0x100, 4, VhostUserConfigFlags::empty(), &buf)
             .unwrap();
         let offset = reply_body.offset;
         assert_eq!(offset, 0x100);
         assert_eq!(&reply_payload, &[0xa5; 4]);
 
-        master.set_slave_request_fd(&eventfd).unwrap();
-        master.set_vring_enable(0, true).unwrap();
+        frontend.set_backend_request_fd(&eventfd).unwrap();
+        frontend.set_vring_enable(0, true).unwrap();
 
-        master
+        frontend
             .set_log_base(
                 0,
                 Some(VhostUserDirtyLogRegion {
@@ -471,10 +471,10 @@ mod tests {
                 }),
             )
             .unwrap();
-        master.set_log_fd(eventfd.as_raw_fd()).unwrap();
+        frontend.set_log_fd(eventfd.as_raw_fd()).unwrap();
 
-        master.set_vring_num(0, 256).unwrap();
-        master.set_vring_base(0, 0).unwrap();
+        frontend.set_vring_num(0, 256).unwrap();
+        frontend.set_vring_base(0, 0).unwrap();
         let config = VringConfigData {
             queue_max_size: 256,
             queue_size: 128,
@@ -484,20 +484,20 @@ mod tests {
             avail_ring_addr: 0x3000,
             log_addr: Some(0x4000),
         };
-        master.set_vring_addr(0, &config).unwrap();
-        master.set_vring_call(0, &eventfd).unwrap();
-        master.set_vring_kick(0, &eventfd).unwrap();
-        master.set_vring_err(0, &eventfd).unwrap();
+        frontend.set_vring_addr(0, &config).unwrap();
+        frontend.set_vring_call(0, &eventfd).unwrap();
+        frontend.set_vring_kick(0, &eventfd).unwrap();
+        frontend.set_vring_err(0, &eventfd).unwrap();
 
-        let max_mem_slots = master.get_max_mem_slots().unwrap();
+        let max_mem_slots = frontend.get_max_mem_slots().unwrap();
         assert_eq!(max_mem_slots, 32);
 
         let region_file: File = TempFile::new().unwrap().into_file();
         let region =
             VhostUserMemoryRegionInfo::new(0x10_0000, 0x10_0000, 0, 0, region_file.as_raw_fd());
-        master.add_mem_region(&region).unwrap();
+        frontend.add_mem_region(&region).unwrap();
 
-        master.remove_mem_region(&region).unwrap();
+        frontend.remove_mem_region(&region).unwrap();
 
         mbar.wait();
     }
@@ -514,8 +514,8 @@ mod tests {
     #[test]
     fn test_should_reconnect() {
         assert!(Error::PartialMessage.should_reconnect());
-        assert!(Error::SlaveInternalError.should_reconnect());
-        assert!(Error::MasterInternalError.should_reconnect());
+        assert!(Error::BackendInternalError.should_reconnect());
+        assert!(Error::FrontendInternalError.should_reconnect());
         assert!(!Error::InvalidParam.should_reconnect());
         assert!(!Error::InvalidOperation("reason").should_reconnect());
         assert!(
